import isNode from '../../helpers/isNode';
import stringOrIntToMs from '../../helpers/stringOrIntToMs';
import { inMemory } from './handlers/inMemory';
import { inRedis } from './handlers/inRedis';
import LRU from './algorithms/LRU';
import RedisLRU from './algorithms/Redis';
var ALGORITHMS = {
  LRU: 'lru'
};
var DEFAULT_TTL = 500;
var caches = {};

var _cache = function _cache(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      algorithm = _ref.algorithm,
      fnName = _ref.fnName,
      instance = _ref.instance,
      original = _ref.original,
      size = _ref.size,
      target = _ref.target,
      ttl = _ref.ttl,
      redis = _ref.redis;

  var cacheKey = target.constructor.name + "::" + fnName;
  var cache = caches[cacheKey];

  if (!cache) {
    if (!redis) {
      cache = new LRU({
        size: size
      });
    } else {
      if (!isNode) {
        console.warn('[sui-decorators/cache] Your redis config will be ignored in client side. Using the default inMemory LRU strategy.');
        cache = new LRU({
          size: size
        });
      } else {
        cache = new RedisLRU({
          size: size,
          redisConnection: redis,
          namespace: cacheKey,
          ttl: ttl
        });
        console.warn("[sui-decorators/cache] You are using redis cache for cacheKey: " + cacheKey + ", your method MUST return a promise");
        return inRedis(target, cache, original, fnName, instance, ttl);
      }
    }
  }

  return inMemory(target, cache, original, fnName, instance, ttl);
};

export default (function (_temp2) {
  var _ref2 = _temp2 === void 0 ? {} : _temp2,
      _ref2$ttl = _ref2.ttl,
      ttl = _ref2$ttl === void 0 ? DEFAULT_TTL : _ref2$ttl,
      _ref2$server = _ref2.server,
      server = _ref2$server === void 0 ? false : _ref2$server,
      _ref2$algorithm = _ref2.algorithm,
      algorithm = _ref2$algorithm === void 0 ? ALGORITHMS.LRU : _ref2$algorithm,
      size = _ref2.size,
      redis = _ref2.redis;

  var timeToLife = stringOrIntToMs({
    ttl: ttl
  }) || DEFAULT_TTL;
  return function (target, fnName, descriptor) {
    if (typeof window !== 'undefined' && window.__SUI_CACHE_DISABLED__ || typeof global !== 'undefined' && global.__SUI_CACHE_DISABLED__) {
      return descriptor;
    } // if we're on node but the decorator doesn't have the server flag
    // then we ignore the usage of the decorator and thus the cache


    if (isNode && !server) {
      return descriptor;
    }

    if (!isNode && redis) {
      console.warn('[sui-decorators/cache] Your redis config will be ignored in client site. Using the default LRU strategy.');
    }

    var configurable = descriptor.configurable,
        enumerable = descriptor.enumerable,
        writable = descriptor.writable;
    var originalGet = descriptor.get;
    var originalValue = descriptor.value;
    var isGetter = !!originalGet; // https://github.com/jayphelps/core-decorators.js/blob/master/src/autobind.js

    return Object.assign({}, {
      configurable: configurable,
      enumerable: enumerable,
      get: function get() {
        var fn = isGetter ? originalGet.call(this) : originalValue;

        if (this === target) {
          return fn;
        }

        var _fnCached = _cache({
          algorithm: algorithm,
          fnName: fnName,
          instance: this,
          original: fn,
          size: size,
          target: target,
          ttl: timeToLife,
          redis: redis
        });

        Object.defineProperty(this, fnName, {
          configurable: configurable,
          writable: writable,
          enumerable: enumerable,
          value: _fnCached
        });
        return _fnCached;
      },
      set: function set(newValue) {
        Object.defineProperty(this, fnName, {
          configurable: true,
          writable: true,
          enumerable: true,
          value: newValue
        });
        return newValue;
      }
    });
  };
});