import isPromise from '../helpers/isPromise';

var defaultErrorHandler = function defaultErrorHandler(err) {
  console.error(err);
};

var dispatchToListeners = function dispatchToListeners(_ref) {
  var onError = _ref.onError,
      onNext = _ref.onNext,
      params = _ref.params,
      result = _ref.result;

  if (isPromise(result)) {
    result.then(function (value) {
      if (value.__INLINE_ERROR__) {
        var error = value[0],
            val = value[1];
        return !error ? onNext({
          params: params,
          result: val
        }) : onError({
          params: params,
          error: error
        });
      }

      onNext({
        params: params,
        result: value
      });
    })["catch"](function (error) {
      return onError({
        params: params,
        error: error
      });
    });
  } else if (result) {
    if (result.__INLINE_ERROR__) {
      var error = result[0],
          val = result[1];
      return !error ? onNext({
        params: params,
        result: val
      }) : onError({
        params: params,
        error: error
      });
    }

    onNext({
      params: params,
      result: result
    });
  }
};

var createSubscription = function createSubscription(proto, method, originalMethod) {
  var onNextListeners = [];
  var onErrorListeners = [];

  proto[method] = function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var params = args;

    try {
      var result = originalMethod.apply(this, args);
      dispatchToListeners({
        onError: onError,
        onNext: onNext,
        params: params,
        result: result
      });
      return result;
    } catch (error) {
      onError({
        params: params,
        error: error
      });
      throw error;
    }
  };

  var onNext = function onNext(_ref2) {
    var params = _ref2.params,
        result = _ref2.result;
    onNextListeners.forEach(function (listener) {
      return listener({
        params: params,
        result: result
      });
    });
  };

  var onError = function onError(_ref3) {
    var params = _ref3.params,
        error = _ref3.error;
    onErrorListeners.forEach(function (listener) {
      return listener({
        params: params,
        error: error
      });
    });
  };

  var subscribe = function subscribe(onNext, onError) {
    if (onNext === void 0) {
      onNext = function onNext() {};
    }

    if (onError === void 0) {
      onError = defaultErrorHandler;
    }

    onNextListeners.push(onNext);
    onErrorListeners.push(onError);
    return {
      dispose: function dispose() {
        onNextListeners = onNextListeners.filter(function (l) {
          return l !== onNext;
        });
        onErrorListeners = onErrorListeners.filter(function (l) {
          return l !== onError;
        });
      }
    };
  };

  return {
    subscribe: subscribe
  };
};

var reducer = function reducer(Target, proto, method) {
  var originalMethod = Target.prototype[method];
  proto.$ = proto.$ || {};
  proto.$[method] = createSubscription(proto, method, originalMethod);
  return proto;
};

export default (function () {
  for (var _len2 = arguments.length, methods = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    methods[_key2] = arguments[_key2];
  }

  return function (Target) {
    Target.prototype.__STREAMIFY__ = true;
    Object.assign(Target.prototype, methods.filter(function (method) {
      return !!Target.prototype[method];
    }).reduce(reducer.bind(null, Target), {}));
    return Target;
  };
});